""" Contains a list of manual translations. Those are usually messages that
exist in external passages which are not queries for i18n extraction.

"""

from onegov.town import _

# messages defined in wtforms-components
_(u"Not a valid color.")

# messages defined in onegov.form
_(u"Uploaded file")
_(u"Keep file")
_(u"Replace file")
_(u"Delete file")
_(u"The file is too large, please provide a file smaller than {}.")
_(u"Files of this type are not supported.")
_(u"The form could not be parsed.")
_(u"Define at least one required e-mail field ('E-Mail * = @@@')")
_(u"The syntax on line {line} is not valid.")
